// WaveElement.cpp: implementation of the WaveElement class.
//
//////////////////////////////////////////////////////////////////////
#include "WaveElement.h"
#include <string.h>
#include <math.h>


WaveElement::WaveElement()
{
	int i;

	strcpy(Name,"Daubechies-4");
	for(i = 0; i < 25; i++)
	{
		Hfilt[i] = (float)0.0;
		Htfilt[i] = (float)0.0;
	}

	Hfilt[0] = (float)SQRT2 * (1 + SQRT3) / 8;  Htfilt[0] = Hfilt[0];
	Hfilt[1] = (float)SQRT2 * (3 + SQRT3) / 8;  Htfilt[1] = Hfilt[1];
	Hfilt[2] = (float)SQRT2 * (3 - SQRT3) / 8;  Htfilt[2] = Hfilt[2];
	Hfilt[3] = (float)SQRT2 * (1 - SQRT3) / 8;  Htfilt[3] = Hfilt[3];

	offH = 1;
	offG = 1;
	offHt = 1;
	offGt = 1;
	imsG = 1;
	imsGt = 1;
	lenH = 4;
	lenHt = 4;
}

WaveElement::WaveElement(char str[], float f1[], float f2[],
						 int o1, int o2, int o3, int o4,
						 int s1, int s2)
{
	int i;

	strcpy(Name,str);
	lenH = sizeof(f1) / sizeof(float);
	lenHt = sizeof(f2) / sizeof(float);

	for(i = 0; i < lenH; i++)
		Hfilt[i] = f1[i];

	for(i = 0; i < lenHt; i++)
		Htfilt[i] = f2[i];

	offH = o1;
	offG = o2;
	offHt = o3;
	offGt = o4;
	imsG = s1;
	imsGt = s2;
}

WaveElement::WaveElement(char str[], float f1[], float f2[],
						 int o1, int o2, int o3, int o4,
						 int s1, int s2, short l1, short l2)
{
	int i;

	strcpy(Name, str);
	lenH = l1;
	lenHt = l1;

	for(i = 0; i < lenH; i++)
		Hfilt[i] = f1[i];

	for(i = 0; i < lenHt; i++)
		Htfilt[i] = f2[i];

	offH = o1;
	offG = o2;
	offHt = o3;
	offGt = o4;
	imsG = s1;
	imsGt = s2;
}

int WaveElement::Mod(int n, int m)
{
	n = n % m;
	if(n < 0)
		n += m;

	return n;
}

void WaveElement::Decompose(float inp[], float out[])
{
	int i, j, pos;
	int res, lh, lg, sg;
	float sum;
	float tmp[PktSize];

	res = PktSize;

	lh = lenH;
	lg = lenHt;

	for(i = res-1; i>= 0; i--)
	{
		out[i] = inp[i];		// initial copying
		tmp[i] = (float)0.0;	// get rid of garbage
	}

	for(res /= 2; res > 0; res /= 2)
	{
		for(i = 0; i < res; i++)
		{
			// H filter
			sum = (float)0.0;
			for(j = 0; j < lh; j++)
			{
				pos = Mod(i+i+j - offH, res+res);
				sum += Hfilt[j] * out[pos];
			}
			tmp[i] = sum;

			// G filter
			sum = (float)0.0;
			sg = imsG;
			for(j=0;j<lg;j++)
			{
				pos = Mod(i+i+j - offG, res+res);
				sum += sg * Htfilt[lg-1-j] * out[pos];
				sg = -sg;
			}
			tmp[i+res] = sum;
		}

		for(i = res+res-1; i>=0; i--)
			out[i] = tmp[i];
	}
}

void WaveElement::Compose(float inp[], float out[])
{
	int i,j;
	int res, resmax, pos, lht, lgt, sgt;
	float tmp[PktSize];
	float val;

	resmax = PktSize >> 1;
	lht = lenHt;
	lgt = lenH;

	for(i = 0; i < PktSize; i++)
		tmp[i] = (float)0.0;

	out[0] = inp[0];
	for(res = 1; res <= resmax; res += res)
	{
		for(i = 0; i < res; i++)
		{
			// H~ filter
			val = out[i];
			for(j = 0; j < lht; j++)
			{
				pos = Mod(i+i+j - offHt, res+res);
				tmp[pos] += Htfilt[j] * val;
			}

			// G~ filter
			sgt = imsGt;
			val = inp[i+res];
			for(j = 0; j < lgt; j++)
			{
				pos = Mod(i+i+j - offGt, res+res);
				tmp[pos] += sgt * Hfilt[lgt-1-j] * val;
				sgt = -sgt;
			}
		}

		for(i = res+res-1; i >= 0; i--)
		{
			out[i] = tmp[i];
			tmp[i] = (float)0.0;
		}
	}
}
