// WaveElement.h: interface for the WaveElement class.
//
//////////////////////////////////////////////////////////////////////

#ifndef WaveElement_H
#define WaveElement_H

#ifndef WaveConstants
#define WaveConstants
	const float SQRT2 = (float)1.41421356237309504880168;   // might be useful
	const float SQRT3 = (float)1.73205080756887729352745;   // for basis defining

	const float fpi = (float)3.1415926535897932384626433;

	const short PktSize = 256;        // Size of packets in # of samples
#endif

#define DllExport __declspec( dllexport )   // Define a macro for exporting to dll


class DllExport WaveElement
{
public:
	WaveElement();     // By default, it's Daubechies-4 wavelet. (2nd-order)

	WaveElement(char str[], float f1[], float f2[],
				int o1, int o2, int o3, int o4,
				int s1, int s2);

	WaveElement(char str[], float f1[], float f2[],
				int o1, int o2, int o3, int o4,
				int s1, int s2,
				short l1, short l2);

	int Mod(int n, int m);
	void Decompose(float inp[], float out[]);		// Wavelet transform
	void Compose(float inp[], float out[]);		// Inverse transform

	char Name[30];         // Filter name -- just to show on the screen.

	float Hfilt[25];       // I'm assuming no farther than 10th order
	float Htfilt[25];      // which at the most has 24 coefficients...
				     // depending on family, anyway.

	int offH, offG, offHt, offGt;   // offsets to look through for the transforms
	int imsG, imsGt;                // initial sign convention
	short lenH, lenHt;              // actual used number of coefficients.
};

#endif